#
# FINEXCA Helper Functions
# Useful Finexca Code
##################################
#


toda.yamamoto <- function(var) {
  # add the magic lag to the existing VAR model
  ty.df <- eval(var$call$y);
  ty.varnames <- colnames(ty.df);
  ty.lags <- var$p + 1;
  ty.augmented_var <- VAR(ty.df, ty.lags, type=var$type, exogen=var$exogen, season=var$season);
  
  ty.results <- data.frame(predictor = character(0), causes = character(0), chisq = numeric(0), p = numeric(0));
  
  for (current_variable in ty.varnames) {
    # construct the restriction matrix: to test if *current_variable* causes any of the others,
    # we test if the lagged values of current variable (ignoring the magic lag) are jointly insignificant
    
    ty.restrictions <- as.matrix(Bcoef(ty.augmented_var))*0+1;
    ty.coefres <- head(grep(current_variable, colnames(ty.restrictions), value=T), -1);
    ty.restrictions[which(rownames(ty.restrictions) != current_variable), ty.coefres] <- 0;
    # estimate restricted var
    ty.restricted_var <- restrict(ty.augmented_var, 'manual', resmat=ty.restrictions);
    
    for (k in 1:length(ty.varnames)) {
      if (ty.varnames[k] != current_variable) {
        my.wald <- waldtest(ty.augmented_var$varresult[[k]], ty.restricted_var$varresult[[k]], test='Chisq');
        ty.results <- rbind(ty.results, data.frame(
          predictor = current_variable, 
          causes = ty.varnames[k], 
          chisq = as.numeric(my.wald$Chisq[2]), 
          p = my.wald$`Pr(>Chisq)`[2])
        );
      }
    }
  }
  return(ty.results);
}

# xtable without rownames
xtable.nr <- function(...) {
  print(do.call(xtable, list(...)),include.rownames=F);
}
options(xtable.comment = FALSE);

# plot acf with ggplot
ggacf <- function(series) {
  significance_level <- qnorm((1 + 0.95)/2)/sqrt(sum(!is.na(series)))  
  a<-acf(series, plot=F)
  a.2<-with(a, data.frame(lag, acf))
  g<- ggplot(a.2[-1,], aes(x=lag,y=acf)) + geom_bar(stat = "identity", position = "identity") + xlab('Lag') + ylab('ACF') +
    geom_hline(yintercept=c(significance_level,-significance_level), lty=3);
  # fix scale for integer lags
  if (all(a.2$lag%%1 == 0)) {
    g<- g + scale_x_discrete(limits = seq(1, max(a.2$lag)));
  }
  return(g);
}


# plot partial autocorrelation function with ggplot
ggpacf <- function(series) {
  significance_level <- qnorm((1 + 0.95)/2)/sqrt(sum(!is.na(series)))  
  a<-pacf(series, plot=F);
  a.mat <- data.frame(acf = a$acf, lag=1:length(a$acf))
  g<- ggplot(a.mat, aes(x=lag,y=acf)) + geom_bar(stat = "identity", position = "identity") + xlab('Lag') + ylab('PACF') +
    geom_hline(yintercept=c(significance_level,-significance_level), lty=3);
  return(g);
}

# lag and pad
lagpad <- function(x, k) {
  if (k>0) {
    return (c(rep(NA, k), x)[1 : length(x)] );
  }
  else {
    return (c(x[(-k+1) : length(x)], rep(NA, -k)));
  }
}

tmp.get_pval <- function(stat, cvs) {
  p<-NA;
  res <- stat >= cvs;
  if (res[3]) {
    p<-0.01;
  } else if (res[2]) {
    p<-0.05;
  } else if (res[1]) {
    p<-0.10;
  };
  return (p-10^(-5));
}

## use texreg to get a table of beta
## example: screenreg(extract.cajo_beta(vecm, vecm.r1))
extract.cajo_beta <- function(cajo, orls, rownames = NA) {
  alpha <- coef(orls$rlm)[1, ];
  resids <- resid(orls$rlm);
  N <- nrow(resids);
  sigma <- crossprod(resids) / N;
  
  # get standard errors and p-values
  beta <- orls$beta
  beta.se <- sqrt(diag(kronecker(solve(crossprod(cajo@RK[, -1])), 
                                 solve(t(alpha) %*% solve(sigma) %*% alpha))));
  beta.se2 <- c(NA, beta.se);
  beta.t <- c(NA, beta[-1] / beta.se);
  beta.pval <- dt(beta.t, df= orls$rlm$df.residual)
  
  # collect johansen results
  c.jo <- matrix(c('$r <= 1$',cajo@teststat[1], NA, tmp.get_pval(cajo@teststat[1], cajo@cval[1,]),
           '$r = 0$', cajo@teststat[2], NA, tmp.get_pval(cajo@teststat[2], cajo@cval[2,]) ), 4)
  if (is.na(rownames)) {
    rownames <- as.character(rownames(beta));
  }
  c.mod <- cbind(rownames, as.numeric(beta), beta.se2, beta.pval)
  c.all <- rbind(t(c.jo), c.mod)
  
  tr <- createTexreg(coef.names = c.all[,1], coef = as.numeric(c.all[,2]), se = as.numeric(c.all[,3]), 
                     pvalues = as.numeric(c.all[,4]),
                     # remove this goodness of fit measure afterwards from the table
                     gof.names = c('Lag Order'), gof = c(cajo@lag), gof.decimal = c(FALSE)
  );
  return(tr);
}

extract.eg <- 
  function (model, include.rsquared = TRUE, include.adjrs = TRUE, 
            include.nobs = TRUE, include.fstatistic = FALSE, include.rmse = TRUE, 
            ...) 
  {
    s <- summary(model, ...)
    names <- rownames(s$coef)
    co <- s$coef[, 1]
    se <- s$coef[, 2]
    pval <- s$coef[, 4]
    rs <- s$r.squared
    adj <- s$adj.r.squared
    n <- nobs(model)
    gof <- numeric()
    gof.names <- character()
    gof.decimal <- logical()
    if (include.rsquared == TRUE) {
      gof <- c(gof, rs)
      gof.names <- c(gof.names, "R$^2$")
      gof.decimal <- c(gof.decimal, TRUE)
    }
    if (include.adjrs == TRUE) {
      gof <- c(gof, adj)
      gof.names <- c(gof.names, "Adj. R$^2$")
      gof.decimal <- c(gof.decimal, TRUE)
    }
    if (include.nobs == TRUE) {
      gof <- c(gof, n)
      gof.names <- c(gof.names, "Num. obs.")
      gof.decimal <- c(gof.decimal, FALSE)
    }
    if (include.fstatistic == TRUE) {
      fstat <- s$fstatistic[[1]]
      gof <- c(gof, fstat)
      gof.names <- c(gof.names, "F statistic")
      gof.decimal <- c(gof.decimal, TRUE)
    }
    if (include.rmse == TRUE && !is.null(s$sigma[[1]])) {
      rmse <- s$sigma[[1]]
      gof <- c(gof, rmse)
      gof.names <- c(gof.names, "RMSE")
      gof.decimal <- c(gof.decimal, TRUE)
    }
    # add engle-granger
    res <- ur.df(residuals(model), selectlags = "BIC")
    gof <- c(gof, res@teststat)
    gof.names <- c(gof.names, "Engle-Yoo Cointegration")
    gof.decimal <- c(gof.decimal, TRUE)
    
    tr <- createTexreg(coef.names = names, coef = co, se = se, 
                       pvalues = pval, gof.names = gof.names, gof = gof, gof.decimal = gof.decimal)
    return(tr)
  }

extract.summary.lm <- function (model, include.rsquared = TRUE, include.adjrs = TRUE, 
                                include.nobs = TRUE, include.fstatistic = FALSE, include.rmse = TRUE, 
                                ...) 
{
  s <- model;
  names <- rownames(s$coef)
  co <- s$coef[, 1]
  se <- s$coef[, 2]
  pval <- s$coef[, 4]
  rs <- s$r.squared
  adj <- s$adj.r.squared
  n <- length(s$residuals)
  gof <- numeric()
  gof.names <- character()
  gof.decimal <- logical()
  if (include.rsquared == TRUE) {
    gof <- c(gof, rs)
    gof.names <- c(gof.names, "R$^2$")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  if (include.adjrs == TRUE) {
    gof <- c(gof, adj)
    gof.names <- c(gof.names, "Adj. R$^2$")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  if (include.nobs == TRUE) {
    gof <- c(gof, n)
    gof.names <- c(gof.names, "Num. obs.")
    gof.decimal <- c(gof.decimal, FALSE)
  }
  if (include.fstatistic == TRUE) {
    fstat <- s$fstatistic[[1]]
    gof <- c(gof, fstat)
    gof.names <- c(gof.names, "F statistic")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  if (include.rmse == TRUE && !is.null(s$sigma[[1]])) {
    rmse <- s$sigma[[1]]
    gof <- c(gof, rmse)
    gof.names <- c(gof.names, "RMSE")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  tr <- createTexreg(coef.names = names, coef = co, se = se, 
                     pvalues = pval, gof.names = gof.names, gof = gof, gof.decimal = gof.decimal)
  return(tr)
}
setMethod("extract",  signature = 'summary.lm', definition = extract.summary.lm)

## helper function: extract robust se for texreg
extract.robust.lm <- function (model, include.rsquared = TRUE, include.adjrs = TRUE, 
                               include.nobs = TRUE, include.fstatistic = FALSE, include.rmse = TRUE, 
                               include.ecm_diag = FALSE, include.adl_diag = FALSE, include.df_diag = FALSE,
                               ...)
{
  ct <- coeftest(model, vcov = vcovHC(model, "HC1"))
  co <- ct[, 1]
  se <- ct[, 2]
  pval <- ct[, 4]
  
  s <- summary(model, ...)
  names <- rownames(s$coef)
  
  # add "coefficients" with ECM diagnostic tests so we can haz stars
  if (include.ecm_diag == TRUE) {
    b <- list(Box.test(residuals(model), lag = 1, type = "Ljung"),
              Box.test(residuals(model), lag = 12, type = "Ljung"),
              Box.test(residuals(model)**2, lag = 1, type = "Ljung"),
              Box.test(residuals(model)**2, lag = 12, type = "Ljung"),
              bgtest(model),
              tseries::jarque.bera.test(residuals(model)),
              bptest(model, varformula = ~ predict(model)),
              bptest(model, varformula = ~ index(model$model))
    );
    names <- c(names, 'Box-Ljung AC 1', 'Box-Ljung AC 12', 'Box-Ljung HET 1', 'Box-Ljung HET 12','Breusch-Godfrey AC1','Jarque-Bera','Breusch-Pagan Fitted', 'Breusch-Pagan Index')
    co <- c(co, b[[1]]$statistic, b[[2]]$statistic, b[[3]]$statistic, b[[4]]$statistic, b[[5]]$statistic, b[[6]]$statistic, b[[7]]$statistic, b[[8]]$statistic)
    se <- c(se, NA, NA, NA, NA, NA, NA, NA, NA)
    pval <- c(pval, b[[1]]$p.value, b[[2]]$p.value, b[[3]]$p.value, b[[4]]$p.value, b[[5]]$p.value, b[[6]]$p.value, b[[7]]$p.value, b[[8]]$p.value)
  }
  
  if (include.adl_diag == TRUE) {
    lr <- -coef(model)[1:2]/coef(model)[3]
    names <- c(names, 'Long-Run Intercept', 'Long-Run Coef')
    co <- c(co, lr[[1]], lr[[2]]);
    pval <- c(pval, NA, NA);
    se <- c(se, NA, NA)
  }
  
  if (include.df_diag == TRUE) {
    z<-ur.df(residuals(model), selectlags = 'BIC', lags = 12)
    names <- c(names, 'ADF Stat', 'ADF Lags (BIC)')
    co <- c(co, z@teststat, sum(str_detect(rownames(coef(z@testreg)), 'diff.lag')))
    pval <- c(pval, NA, NA)
    se <- c(se, NA, NA)
  }

  rs <- s$r.squared
  adj <- s$adj.r.squared
  n <- nobs(model)
  gof <- numeric()
  gof.names <- character()
  gof.decimal <- logical()
  if (include.rsquared == TRUE) {
    gof <- c(gof, rs)
    gof.names <- c(gof.names, "R$^2$")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  if (include.adjrs == TRUE) {
    gof <- c(gof, adj)
    gof.names <- c(gof.names, "Adj. R$^2$")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  if (include.nobs == TRUE) {
    gof <- c(gof, n)
    gof.names <- c(gof.names, "Num. obs.")
    gof.decimal <- c(gof.decimal, FALSE)
  }
  if (include.fstatistic == TRUE) {
    fstat <- s$fstatistic[[1]]
    gof <- c(gof, fstat)
    gof.names <- c(gof.names, "F statistic")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  if (include.rmse == TRUE && !is.null(s$sigma[[1]])) {
    rmse <- s$sigma[[1]]
    gof <- c(gof, rmse)
    gof.names <- c(gof.names, "RMSE")
    gof.decimal <- c(gof.decimal, TRUE)
  }
  tr <- createTexreg(coef.names = names, coef = co, se = se, 
                     pvalues = pval, gof.names = gof.names, gof = gof, gof.decimal = gof.decimal)
  return(tr)
}

theme_finexca <- theme_bw() + theme(legend.position="bottom", legend.title=element_blank(), legend.margin=margin(t = -0.5, unit='cm'));
